@@ -631,20 +631,6 @@ class DbaseFile:
                 res.append(result)
         return res
 
-    def as_cursor(self, records:List[Record]=None, fields:List[DbaseField]=None,
-                  start:int=0, stop:int=None, step:int=1):
-        """
-        Returns a cursor object for the database.
-        """
-        if not fields:
-            fields = self.fields
-        if not records:
-            records = self
-
-        description = [field.alias for field in fields]
-        records = (self.transform(r, fields) for r in records[start:stop:step])
-        return Cursor(description, records)
-    
     def commit(self, filename=None):
         """
         Writes the database to a file. 
@@ -839,6 +825,8 @@ class DbaseFile:
             f.name.strip() == fieldname.strip()
             or f.alias.strip() == fieldname.strip()
             )
+        if isinstance(fieldname, DbaseField):
+            return fieldname
         for field in self.fields:
             if cond_true(field):
                 return field
@@ -968,7 +956,7 @@ class DbaseFile:
         index = -1
         while True:
             index, record = self.search(fieldname, value, index + 1, "", compare_function)  
-            if index < 0 or index >= self.header.records - 1:
+            if index < 0 or index >= self.header.records:
                 return ret
             else:    
                 ret.append(record)
@@ -1003,7 +991,7 @@ class DbaseFile:
     @staticmethod
     def _format_field(field, record):
         if field.type == FieldType.CHARACTER.value:
-            return str(record.get(field.name) or '').ljust(field.length + 2)
+            return str(record.get(field.name) or record.get(field.alias) or '').ljust(field.length + 2)
         else: 
             return str(record.get(field.name) or '').rjust(field.length + 2)
     
@@ -1108,6 +1096,30 @@ class DbaseFile:
                 raise ValueError(f"Unknown field type {field.type}")
         self.file.flush()
 
+    def transform(self, record:Record, fields:List[DbaseField]):
+        ret = Record()
+        for field in [self.get_field(f) for f in fields]:
+            # ret[fields[field]] = record.get(field.name)
+            ret[field.alias] = record.get(field.name) or record.get(field.alias)
+        return ret
+
+    def as_cursor(self, records:List[Record]=None, fields:List[DbaseField]=None,
+                  start:int=0, stop:int=None, step:int=1):
+        """
+        Returns a cursor object for the database.
+        """
+        if not fields:
+            fields = self.fields
+        else:
+            fields = [self.get_field(field) for field in fields]
+        if not records:
+            records = self[start:stop:step]
+
+        description = [(i, field.alias, field.name, field.type, 
+                        field.length, field.decimal) for i, field in enumerate(fields)]
+        records = (self.transform(r, fields) for r in records)
+        return Cursor(description, records)
+
     def execute(self, sql_cmd: str):
         """
         Executes a SQL command on the database.
@@ -1125,22 +1137,20 @@ class DbaseFile:
             if field not in self.field_names:
                 raise ValueError(f"Field {field} not found")
             fieldobjs = {**fieldobjs, **{field: sql_parser.fields[field]}}
-        records = self.fields_view(fields=fieldobjs, 
-                                   records=self.filter(sql_parser.field_param, 
-                                                       sql_parser.value_param,
-                                                       compare_function=sql_parser.compare_function))
-        cursor = Cursor([fieldobjs[k] for k in fieldobjs], records)
+        selectedfields = []
+        for field in fieldobjs:
+            f = self.get_field(field)
+            f.alias = fieldobjs[field]
+            selectedfields.append(f)
+        filteredrecords = self.filter(sql_parser.field_param, 
+                                      sql_parser.value_param,
+                                      compare_function=sql_parser.compare_function)
+        records = self.fields_view(fields=selectedfields, records=filteredrecords)
+        cursor = Cursor(description=[(i, f.alias, f.name, f.type, f.length, f.decimal) 
+                                     for i, f in enumerate(selectedfields)], 
+                                     records=records)
         return cursor
 
-    @staticmethod
-    def transform(record:Record, fields:List[DbaseField]):
-        ret = Record()
-        for field in fields:
-            # ret[fields[field]] = record.get(field.name)
-            ret[field.alias] = record.get(field.name) or record.get(field.alias)
-        return ret
-
-
     def fields_view(self, start=0, stop=None, step=1, fields:List[DbaseField]=None, records=None):
         """
         Returns a generator yielding a record with fields specified in the fields dictionary.
@@ -1308,7 +1318,7 @@ class SQLParser:
         self._sql = sqlstr
         self._type:SQLType = None
 
-        self.fields:SmartDict = None
+        self.fields:List[DbaseField] = None
         self.tables:List[AnyStr] = None
         self.compare_func_src:str = None
         self.function:Callable = None
@@ -1373,7 +1383,7 @@ class SQLParser:
                 self._selectsrc = d.get('selectsrc').strip()
                 self._selend = d.get('selend').strip()
             else:
-                raise ValueError(f"Invalid SELECT command: '{self._sql}'")
+                raise ValueError(f"Invalid SELECT clause: '{self._sql}'")
             if self._selend != ';':
                 m = self.from_re.match(self._sql.strip())
                 if m:
@@ -1381,7 +1391,7 @@ class SQLParser:
                     self._fromsrc = d.get('fromsrc').strip()
                     self._fromend = d.get('fromend').strip()
                 else:
-                    raise ValueError(f"Invalid FROM command: '{self._sql}'")
+                    raise ValueError(f"Invalid FROM clause: '{self._sql}'")
                 if self._fromend != ';':
                     m = self.where_re.match(self._sql.strip())
                     if m:
@@ -1389,12 +1399,16 @@ class SQLParser:
                         self._wheresrc = d.get('wheresrc').strip()
                         self._whereend = d.get('whereend').strip()
                     else:
-                        raise ValueError(f"Invalid WHERE command: '{self._sql}'")   
+                        raise ValueError(f"Invalid WHERE clause: '{self._sql}'")   
                 else:
                     self._wheresrc = self._whereend = None
             else:
                 self._fromsrc = self._fromend = self._wheresrc = self._whereend = None
 
+            if self._fromsrc:
+                self.tables = self._fromsrc.split(',')
+                self.tables = [table.strip() for table in self.tables]
+
             if self._selectsrc:
                 fields = self._selectsrc.split(',')
                 fields = [field.strip() for field in fields]
@@ -1412,10 +1426,6 @@ class SQLParser:
                         break
                 self.fields = dfields
 
-            if self._fromsrc:
-                self.tables = self._fromsrc.split(',')
-                self.tables = [table.strip() for table in self.tables]
-
             if self._wheresrc:
                 self.field_param, self.operator, self.value_param = self.parse_where_clause(self._wheresrc)
                 self.compare_function, self.compare_func_src = self.compile_where_clause(self.field_param, self.operator, self.value_param)
@@ -1438,25 +1448,24 @@ class SQLParser:
     
 @dataclass
 class Cursor:
-    description: List[AnyStr] = field(default_factory=list)
+    description: List[Tuple[int, str, str, str, int, int]] = field(default_factory=list)
     records: Generator = None
 
-    # def __post_init__(self):
-    #     self._init()
-
-    # def _init(self):
-    #     """
-    #     Initializes fields.
-    #     """
-    #     self.description = []
-    #     self.records = None
+    def __init__(self, description:List[Tuple[str, str, str, int, int]]=None, 
+                 records:List[Record]=None, **kwargs):
+        self.description = description or [] 
+        self.records = records
+        if '_connection' in kwargs:
+            self._connection = kwargs['_connection']
+        else:
+            self._connection = None
 
     def fetchone(self):
         """
         Returns the next record from the cursor.
         """
         try:
-            return next(self.records)
+            return next(self.records) if self.records else None
         except StopIteration:
             return None
 
@@ -1464,7 +1473,7 @@ class Cursor:
         """
         Returns all records from the cursor.
         """
-        return list(self.records)
+        return list(self.records) if self.records else []
 
     def fetchmany(self, size):
         """
@@ -1479,6 +1488,21 @@ class Cursor:
                 break
         return retval
 
+    def execute(self, sql:str):
+        if not self._connection:
+            raise ValueError("No connection, cannot execute SQL command")
+        return self._connection.execute(sql)
+    
+        # sql_parser = SQLParser(sql)
+        # if sql_parser.type != 'SELECT':
+        #     raise ValueError("Only SELECT commands are supported right now.")
+        # for i, table in enumerate(self._connection.tables):
+        #     if table == sql_parser.tables[0]:
+        #         dbf = DbaseFile(self._connection.filenames[i])
+        #         cursor = dbf.execute(sql)
+        #         return cursor
+        # raise ValueError(f"Table '{sql_parser.tables[0]}' not found")
+ 
 
 class Connection:
     def __init__(self, dirname:str):
@@ -1514,11 +1538,140 @@ class Connection:
     def filenames(self):
         return self._files
     
+    def Cursor(self):
+        return Cursor(_connection=self)
+    
+    def execute(self, sql:str):
+        sql_parser = SQLParser(sql)
+        if sql_parser.type != 'SELECT':
+            raise ValueError("Only SELECT commands are supported right now.")
+        for i, table in enumerate(self.tables):
+            if table == sql_parser.tables[0]:
+                dbf = DbaseFile(self.filenames[i])
+                cursor = dbf.execute(sql)
+                return cursor
+        raise ValueError(f"Table '{sql_parser.tables[0]}' not found")
+
+def connect(dirname:str):
+    return Connection(dirname)
+
+toplines = {
+    'pretty_table': {'left': BoxType.UP_L.value, 
+                     'join': BoxType.T_DOWN.value,
+                      'line': BoxType.HORZ.value,
+                      'right': BoxType.UP_R.value
+                    },
+    'table': {'left': '+', 'join': '+', 'line': '-', 'right': '+'},
+    'line':  {'left': ' ', 'join': ' ', 'line': ' ', 'right': ' '},
+    'list':  {'left': ' ', 'join': ' ', 'line': ' ', 'right': ' '},
+    'csv':   {'left': ' ', 'join': ' ', 'line': ' ', 'right': ' '},
+}
+
+seplines = {
+    'pretty_table': {'left': BoxType.T_L.value, 
+                     'join': BoxType.CROSS.value,
+                      'line': BoxType.HORZ.value,
+                      'right': BoxType.T_R.value
+                    },
+    'table': {'left': '+', 'join': '+', 'line': '-', 'right': '+'},
+    'line':  {'left': ' ', 'join': ' ', 'line': ' ', 'right': ' '},
+    'list':  {'left': ' ', 'join': ' ', 'line': ' ', 'right': ' '},
+    'csv':   {'left': ' ', 'join': ' ','line': ' ', 'right': ' '},
+}
+
+bottomlines = {
+    'pretty_table': {'left': BoxType.DOWN_L.value, 
+                     'join': BoxType.T_UP.value,
+                      'line': BoxType.HORZ.value,
+                      'right': BoxType.DOWN_R.value
+                    },
+    'table': {'left': '+', 'join': '+', 'line': '-', 'right': '+'},
+    'line':  {'left': ' ', 'join': ' ', 'line': ' ', 'right': ' '},
+    'list':  {'left': ' ', 'join': ' ', 'line': ' ', 'right': ' '},
+    'csv':   {'left': ' ', 'join': ' ', 'line': ' ', 'right': ' '},
+}
+
+separators = {
+    'pretty_table': BoxType.VERT.value,
+    'table': '|',
+    'line':  ' ',
+    'list':  '|',
+    'csv':   ',',
+}
+
+def make_topline(linetype:str, description: List[Tuple[int, str, str, str, int, int]])-> str:
+    d = toplines[linetype]
+    string = d['left']
+    string += d['join'].join([d['line'] * max([t[4], len(t[1])]) for t in description])
+    string += d['right']
+    return string
+
+def make_bottomline(linetype:str, description: List[Tuple[int, str, str, str, int, int]])-> str:
+    d = bottomlines[linetype]
+    string = d['left']
+    string += d['join'].join([d['line'] * max([t[4], len(t[1])]) for t in description])
+    string += d['right']
+    return string
+
+def make_intermediateline(linetype:str, description: List[Tuple[int, str, str, str, int, int]])-> str:
+    d = seplines[linetype]
+    string = d['left']
+    string += d['join'].join([d['line'] * max([t[4], len(t[1])]) for t in description])
+    string += d['right']
+    return string
+
+def make_header_line(linetype:str, description: List[Tuple[int, str, str, str, int, int]])-> str:
+    d = separators[linetype]
+    string = d if linetype  != 'csv' else ' '
+    string += d.join([t[1].center(max([t[4], len(t[1])])) for t in description])
+    string += d if linetype  != 'csv' else ' '
+    return string
+
+def make_cursor_line(linetype:str, r: Record, description: List[Tuple[int, str, str, str, int, int]])-> str:
+    adjstr = lambda v, w:  str(v).rjust(w) if str(v).isdigit() else str(v).ljust(w)
+    d = separators[linetype]
+    string = d if linetype  != 'csv' else ' '
+    string += d.join([adjstr(r[t[1]], max([t[4], len(t[1])])) for t in description])
+    string += d if linetype  != 'csv' else ' '
+    return string
+
+def make_cursor_lines(linetype:str, curr: Cursor)-> Generator[str, None, None]:
+    description = curr.description
+    yield make_topline(linetype, description)
+    yield make_header_line(linetype, description)
+    for record in curr.fetchall():
+        if linetype in ('pretty_table', 'table'):
+            yield make_intermediateline(linetype, description)
+        yield make_cursor_line(linetype, record, description)
+    yield make_bottomline(linetype, description)
+
+def make_raw_lines(curr: Cursor)-> Generator[str, None, None]:
+    return make_cursor_lines('line', curr)
+
+def make_list_lines(curr: Cursor)-> Generator[str, None, None]:
+    return make_cursor_lines('list', curr)
+
+def make_csv_lines(curr: Cursor)-> Generator[str, None, None]:
+    return make_cursor_lines('csv', curr)
+
+def make_table_lines(curr: Cursor)->Generator[str, None, None]:
+    return make_cursor_lines('table', curr)
+
+def make_pretty_table_lines(curr: Cursor)-> Generator[str, None, None]:
+    return make_cursor_lines('pretty_table', curr)
+
 if __name__ == '__main__':
-    from test import test_sql
-    test_sql()
-    os.sys.exit(0)
-    from test import testdb
-    testdb()
-    print("Done!")
+    # teams = DbaseFile('db/teams.dbf')
+    # curr = teams.as_cursor()
+    # from test import test_sql
+    # test_sql()
+    # os.sys.exit(0)
+    teams = DbaseFile('db/teams.dbf')
+    curr = teams.execute("select * from teams where id > 0;")
+    recteams = curr.fetchall()
+    print(f"There are {len(recteams)} teams.")
+    # from test import testdb
+    # testdb()
+    # print("Done!")
+    pass
     
